@@ -8,6 +8,7 @@ this.details = null;
 this.search = null;
 this.jmolAtoms = null;
 this.directives = null;
+this.v = null;
 this.polyhedralOrders = null;
 this.isNot = false;
 this.sorter = null;
@@ -18,9 +19,11 @@ c$.getChiralityClass = Clazz.defineMethod (c$, "getChiralityClass",
 return Clazz.doubleToInt (("0;PH;AL;TP;TH;TB;OH;SP;TS;SS;".indexOf (xx) + 1) / 3);
 }, "~S");
 c$.newStereo = Clazz.defineMethod (c$, "newStereo", 
-function (stereo) {
-return (stereo == null ?  new JS.SmilesStereo (0, 0, 0, null, null) :  new JS.SmilesStereo (stereo.chiralClass, stereo.chiralOrder, stereo.atomCount, stereo.details, stereo.directives));
-}, "JS.SmilesStereo");
+function (search) {
+var stereo =  new JS.SmilesStereo (0, 0, 0, null, null);
+stereo.search = search;
+return stereo;
+}, "JS.SmilesSearch");
 Clazz.makeConstructor (c$, 
 function (chiralClass, chiralOrder, atomCount, details, directives) {
 this.chiralClass = chiralClass;
@@ -142,15 +145,19 @@ throw e;
 return true;
 }, "JS.SmilesAtom");
 Clazz.defineMethod (c$, "setTopoCoordinates", 
- function (atom, sAtom, sAtom2, cAtoms) {
+function (atom, sAtom, sAtom2, cAtoms) {
 var chClass = atom.stereo.chiralClass;
 var chiralOrder = atom.stereo.chiralOrder;
-var a2 = (chClass == 2 ? this.jmolAtoms[sAtom2.getMatchingAtomIndex ()] : null);
 atom.set (0, 0, 0);
+var map;
+if (this.jmolAtoms == null) {
+map =  Clazz.newIntArray (-1, [0, 1, 2, 3]);
+} else {
 atom = this.jmolAtoms[sAtom.getMatchingAtomIndex ()];
 atom.set (0, 0, 0);
-var map = this.getMappedTopoAtoms (atom, a2, cAtoms);
-var pt;
+var a2 = (chClass == 2 ? this.jmolAtoms[sAtom2.getMatchingAtomIndex ()] : null);
+map = this.getMappedTopoAtoms (atom, a2, cAtoms);
+}var pt;
 switch (chClass) {
 case 1:
 break;
@@ -292,6 +299,7 @@ for (var i = bonds.length; --i >= 0; ) bonds[i] = aTemp[i][0];
 }, "JU.SimpleNode,JU.SimpleNode,JU.T3,~A,JU.V3");
 Clazz.defineMethod (c$, "checkStereoChemistry", 
 function (search, v) {
+this.v = v;
 this.search = search;
 this.jmolAtoms = search.targetAtoms;
 var haveTopo = search.haveTopo;
@@ -301,54 +309,66 @@ for (var i = 0; i < search.ac; i++) {
 var pAtom = search.patternAtoms[i];
 if (pAtom.stereo == null) continue;
 var isNot = (pAtom.not != invertStereochemistry);
+var atom0 = pAtom.getMatchingAtom ();
+switch (this.checkStereoForAtom (pAtom, atom0, isNot, haveTopo)) {
+case 0:
+continue;
+case 1:
+return true;
+case -1:
+return false;
+}
+}
+return true;
+}, "JS.SmilesSearch,JS.VTemp");
+Clazz.defineMethod (c$, "checkStereoForAtom", 
+function (pAtom, atom0, isNot, haveTopo) {
 var atom1 = null;
 var atom2 = null;
 var atom3 = null;
 var atom4 = null;
 var atom5 = null;
 var atom6 = null;
-var pAtom1 = null;
 var pAtom2 = null;
 var sAtom0 = null;
 var jn;
-var atom0 = pAtom.getMatchingAtom ();
 if (haveTopo) sAtom0 = atom0;
 var nH = Math.max (pAtom.explicitHydrogenCount, 0);
 var order = pAtom.stereo.chiralOrder;
 var chiralClass = pAtom.stereo.chiralClass;
-if (haveTopo && sAtom0.getChiralClass () != chiralClass) return false;
+if (haveTopo && sAtom0.getChiralClass () != chiralClass) return -1;
 if (JU.Logger.debugging) JU.Logger.debug ("...type " + chiralClass + " for pattern atom \n " + pAtom + "\n " + atom0);
 switch (chiralClass) {
 case 1:
 if (pAtom.stereo.isNot) isNot = !isNot;
-if (nH > 1 || pAtom.bondCount == 0) continue;
+if (nH > 1 || pAtom.bondCount == 0) return 0;
 if (haveTopo) {
-continue;
+return 0;
 }var bonds = pAtom.bonds;
 var jHpt = -1;
 if (nH == 1) {
 jHpt = (pAtom.isFirst ? 0 : 1);
-if (pAtom.getBondCount () != 3) return false;
-v.vA.set (0, 0, 0);
-for (var j = 0; j < 3; j++) v.vA.add (bonds[j].getOtherAtom (sAtom0).getMatchingAtom ());
+if (pAtom.getBondCount () != 3) return -1;
+this.v.vA.set (0, 0, 0);
+for (var j = 0; j < 3; j++) this.v.vA.add (bonds[j].getOtherAtom (sAtom0).getMatchingAtom ());
 
-v.vA.scale (0.3333);
-v.vA.sub2 (atom0, v.vA);
-v.vA.add (atom0);
+this.v.vA.scale (0.3333);
+this.v.vA.sub2 (atom0, this.v.vA);
+this.v.vA.add (atom0);
 }var po = pAtom.stereo.polyhedralOrders;
 var pt;
 for (var j = po.length; --j >= 0; ) {
 var orders = po[j];
 if (orders == null || orders.length < 2) continue;
 pt = (j > jHpt ? j - nH : j);
-var ta1 = (j == jHpt ? v.vA : bonds[pt].getOtherAtom (pAtom).getMatchingAtom ());
+var ta1 = (j == jHpt ? this.v.vA : bonds[pt].getOtherAtom (pAtom).getMatchingAtom ());
 var flast = (isNot ? 3.4028235E38 : 0);
 var ta2 = null;
 for (var k = 0; k < orders.length; k++) {
 pt = orders[k];
 var ta3;
 if (pt == jHpt) {
-ta3 = v.vA;
+ta3 = this.v.vA;
 } else {
 if (pt > jHpt) pt--;
 ta3 = bonds[pt].getOtherAtom (pAtom).getMatchingAtom ();
@@ -357,67 +377,19 @@ ta2 = ta3;
 continue;
 }var f = JU.Measure.computeTorsion (ta3, ta1, atom0, ta2, true);
 if (Float.isNaN (f)) f = 180;
-if (orders.length == 2) return ((f < 0) != isNot);
+if (orders.length == 2) return ((f < 0) != isNot ? 1 : -1);
 if (f < 0) f += 360;
-if ((f < flast) != isNot) return false;
+if ((f < flast) != isNot) return -1;
 flast = f;
 }
 }
-continue;
+return 0;
 case 2:
-pAtom1 = pAtom.getBond (0).getOtherAtom (pAtom);
-pAtom2 = pAtom.getBond (1).getOtherAtom (pAtom);
-if (pAtom1 == null || pAtom2 == null) continue;
-var pAtom1a = pAtom;
-var pAtom2a = pAtom;
-while (pAtom1.getBondCount () == 2 && pAtom2.getBondCount () == 2 && pAtom1.getValence () == 4 && pAtom2.getValence () == 4) {
-var b = pAtom1.getBondNotTo (pAtom1a, true);
-pAtom1a = pAtom1;
-pAtom1 = b.getOtherAtom (pAtom1);
-b = pAtom2.getBondNotTo (pAtom2a, true);
-pAtom2a = pAtom2;
-pAtom2 = b.getOtherAtom (pAtom2);
-}
-pAtom = pAtom1;
-jn =  new Array (6);
-jn[4] =  new JS.SmilesAtom ().setIndex (60004);
-var nBonds = pAtom.getBondCount ();
-if (nBonds != 2 && nBonds != 3) continue;
-for (var k = 0, p = 0; k < nBonds; k++) {
-var b = pAtom.bonds[k];
-pAtom1 = b.getOtherAtom (pAtom);
-if (b.matchingBond.getCovalentOrder () == 2) {
-if (pAtom2 == null) pAtom2 = pAtom1;
-continue;
-}if (b.atom1 === pAtom1) {
-p = 0;
-} else if (jn[1] == null) {
-p = 1;
-} else {
-jn[0] = jn[p = 1];
-}jn[p] = pAtom1.getMatchingAtom ();
-}
-if (pAtom2 == null) continue;
-nBonds = pAtom2.getBondCount ();
-if (nBonds != 2 && nBonds != 3) continue;
-for (var p = 0, k = 0; k < nBonds; k++) {
-var b = pAtom2.bonds[k];
-pAtom1 = b.getOtherAtom (pAtom2);
-if (b.atom1 === pAtom1) {
-p = 2;
-} else if (jn[3] == null) {
-p = 3;
-} else {
-jn[2] = jn[p = 3];
-}jn[p] = pAtom1.getMatchingAtom ();
-}
-for (var k = 0; k < 2; k++) if (jn[k] == null) this.getAlleneAtom (pAtom, jn, k, haveTopo);
-
-for (var k = 2; k < 4; k++) if (jn[k] == null) this.getAlleneAtom (pAtom2, jn, k, haveTopo);
-
-if (haveTopo && !this.setTopoCoordinates (sAtom0, pAtom, pAtom2, jn)) return false;
-if (!JS.SmilesStereo.checkStereochemistryAll (pAtom.not != invertStereochemistry, atom0, chiralClass, order, jn[0], jn[1], jn[2], jn[3], null, null, v)) return false;
-continue;
+jn = this.getAlleneAtoms (pAtom, null);
+if (jn == null) return 0;
+if (haveTopo && !this.setTopoCoordinates (sAtom0, pAtom, pAtom2, jn)) return -1;
+if (!JS.SmilesStereo.checkStereochemistryAll (isNot, atom0, chiralClass, order, jn[0], jn[1], jn[2], jn[3], null, null, this.v)) return -1;
+return 0;
 case 8:
 case 9:
 case 3:
@@ -431,28 +403,82 @@ case 0:
 atom2 = this.getJmolAtom (pAtom.getMatchingBondedAtom (1));
 break;
 case 1:
-atom2 = search.findImplicitHydrogen (pAtom.getMatchingAtom ());
+atom2 = this.search.findImplicitHydrogen (pAtom.getMatchingAtom ());
 if (pAtom.isFirst) {
 var a = atom2;
 atom2 = atom1;
 atom1 = a;
 }break;
 default:
-continue;
+return 0;
 }
 atom3 = this.getJmolAtom (pAtom.getMatchingBondedAtom (2 - nH));
 atom4 = this.getJmolAtom (pAtom.getMatchingBondedAtom (3 - nH));
 atom5 = this.getJmolAtom (pAtom.getMatchingBondedAtom (4 - nH));
 atom6 = this.getJmolAtom (pAtom.getMatchingBondedAtom (5 - nH));
-if (haveTopo && !this.setTopoCoordinates (sAtom0, pAtom, null,  Clazz.newArray (-1, [atom1, atom2, atom3, atom4, atom5, atom6]))) return false;
-if (!JS.SmilesStereo.checkStereochemistryAll (isNot, atom0, chiralClass, order, atom1, atom2, atom3, atom4, atom5, atom6, v)) return false;
+if (haveTopo && !this.setTopoCoordinates (sAtom0, pAtom, null,  Clazz.newArray (-1, [atom1, atom2, atom3, atom4, atom5, atom6]))) return -1;
+if (!JS.SmilesStereo.checkStereochemistryAll (isNot, atom0, chiralClass, order, atom1, atom2, atom3, atom4, atom5, atom6, this.v)) return -1;
+return 0;
+}
+return 0;
+}, "JS.SmilesAtom,JU.Node,~B,~B");
+Clazz.defineMethod (c$, "getAlleneAtoms", 
+function (pAtom, pAtom1) {
+if (pAtom1 == null) pAtom1 = pAtom.getBond (0).getOtherAtom (pAtom);
+var pAtom2 = pAtom.getBond (1).getOtherAtom (pAtom);
+if (pAtom2 === pAtom1) pAtom2 = pAtom.getBond (0).getOtherAtom (pAtom);
+if (pAtom1 == null || pAtom2 == null) return null;
+var pAtom1a = pAtom;
+var pAtom2a = pAtom;
+while (pAtom1.getBondCount () == 2 && pAtom2.getBondCount () == 2 && pAtom1.getValence () == 4 && pAtom2.getValence () == 4) {
+var b = pAtom1.getBondNotTo (pAtom1a, true);
+pAtom1a = pAtom1;
+pAtom1 = b.getOtherAtom (pAtom1);
+b = pAtom2.getBondNotTo (pAtom2a, true);
+pAtom2a = pAtom2;
+pAtom2 = b.getOtherAtom (pAtom2);
+}
+pAtom = pAtom1;
+var jn =  new Array (6);
+jn[4] =  new JS.SmilesAtom ().setIndex (60004);
+var nBonds = pAtom.getBondCount ();
+if (nBonds != 2 && nBonds != 3) return null;
+for (var k = 0, p = 0; k < nBonds; k++) {
+var b = pAtom.bonds[k];
+pAtom1 = b.getOtherAtom (pAtom);
+if (b.getMatchingBond ().getCovalentOrder () == 2) {
+if (pAtom2 == null) pAtom2 = pAtom1;
 continue;
+}if (b.atom1 === pAtom1) {
+p = 0;
+} else if (jn[1] == null) {
+p = 1;
+} else {
+jn[0] = jn[p = 1];
+}jn[p] = pAtom1.getMatchingAtom ();
 }
+if (pAtom2 == null) return null;
+nBonds = pAtom2.getBondCount ();
+if (nBonds != 2 && nBonds != 3) return null;
+for (var p = 0, k = 0; k < nBonds; k++) {
+var b = pAtom2.bonds[k];
+pAtom1 = b.getOtherAtom (pAtom2);
+if (b.getMatchingBond ().getCovalentOrder () == 2) {
+continue;
+}if (b.atom1 === pAtom1) {
+p = 2;
+} else if (jn[3] == null) {
+p = 3;
+} else {
+jn[2] = jn[p = 3];
+}jn[p] = pAtom1.getMatchingAtom ();
 }
-return true;
-}, "JS.SmilesSearch,JS.VTemp");
-Clazz.defineMethod (c$, "getAlleneAtom", 
- function (pAtom, jn, k, haveTopo) {
+for (var k = 0; k < 4; k++) if (jn[k] == null) this.addAlleneLonePair (k < 2 ? pAtom : pAtom2, jn, k);
+
+return jn;
+}, "JS.SmilesAtom,JS.SmilesAtom");
+Clazz.defineMethod (c$, "addAlleneLonePair", 
+ function (pAtom, jn, k) {
 var atom = pAtom.getMatchingAtom ();
 jn[k] = this.search.findImplicitHydrogen (atom);
 if (jn[k] != null) return;
@@ -465,7 +491,7 @@ v.setT ((jn[4]));
 v.scaleAdd2 (2, pAtom.getMatchingAtom (), v);
 }jn[k] =  new JS.SmilesAtom ().setIndex (-2147483648);
 (jn[k]).setT (v);
-}, "JS.SmilesAtom,~A,~N,~B");
+}, "JS.SmilesAtom,~A,~N");
 c$.getStereoFlag = Clazz.defineMethod (c$, "getStereoFlag", 
 function (atom0, atoms, nAtoms, v) {
 var atom1 = atoms[0];
@@ -556,7 +582,7 @@ JU.Measure.getNormalThroughPoints (atom2, atom3, atom4, v.vNorm3, v.vTemp1);
 JU.Measure.getNormalThroughPoints (atom3, atom4, atom1, v.vNorm4, v.vTemp1);
 }, "JU.P3,JU.P3,JU.P3,JU.P3,JS.VTemp");
 c$.checkChirality = Clazz.defineMethod (c$, "checkChirality", 
-function (pattern, index, newAtom) {
+function (search, pattern, index, newAtom) {
 var stereoClass = 0;
 var order = -2147483648;
 var len = pattern.length;
@@ -614,11 +640,12 @@ throw e;
 index = pt;
 }}if (order < 1 || stereoClass < 0) throw  new JS.InvalidSmilesException ("Invalid stereochemistry descriptor");
 }newAtom.stereo =  new JS.SmilesStereo (stereoClass, order, atomCount, details, directives);
+newAtom.stereo.search = search;
 if (JS.SmilesParser.getChar (pattern, index) == '?') {
 JU.Logger.info ("Ignoring '?' in stereochemistry");
 index++;
 }return index;
-}, "~S,~N,JS.SmilesAtom");
+}, "JS.SmilesSearch,~S,~N,JS.SmilesAtom");
 Clazz.defineMethod (c$, "getPolyhedralOrders", 
  function () {
 var po = this.polyhedralOrders = JU.AU.newInt2 (this.atomCount);
