@@ -498,10 +498,20 @@ Clazz.overrideMethod (c$, "calculateCIPChiralityForAtoms",
 function (vwr, bsAtoms) {
 vwr.setCursor (3);
 var cip = this.getCIPChirality (vwr);
-var data = (J.api.Interface.getInterface ("JS.CIPData", vwr, "script")).set (vwr, bsAtoms);
+var dataClass = (vwr.getBoolean (603979962) ? "CIPData" : "CIPDataTracker");
+var data = (J.api.Interface.getInterface ("JS." + dataClass, vwr, "script")).set (vwr, bsAtoms);
 cip.getChiralityForAtoms (data);
 vwr.setCursor (0);
 }, "JV.Viewer,JU.BS");
+Clazz.overrideMethod (c$, "calculateCIPChiralityForSmiles", 
+function (vwr, smiles) {
+vwr.setCursor (3);
+var cip = this.getCIPChirality (vwr);
+var data = (J.api.Interface.getInterface ("JS.CIPDataSmiles", vwr, "script")).setAtomsForSmiles (vwr, smiles);
+cip.getChiralityForAtoms (data);
+vwr.setCursor (0);
+return data.getSmilesChiralityArray ();
+}, "JV.Viewer,~S");
 Clazz.defineMethod (c$, "getCIPChirality", 
  function (vwr) {
 return (this.cip == null ? (this.cip = (J.api.Interface.getInterface ("JS.CIPChirality", vwr, "script"))) : this.cip);
